// Copyright (c) 2014 Nora
// Released under the MIT license
// http://opensource.org/licenses/mit-license.php
#ifndef _OVRMIRROR_COMMON_IRENDERER_HPP_
#define _OVRMIRROR_COMMON_IRENDERER_HPP_

#include "Config.hpp"
#include "Thread.hpp"

namespace OVRMirror
{

struct RendererParams
{
	RendererParams()
	{
		device = nullptr;
		window = nullptr;
		memset( &backBufferSize, 0, sizeof(backBufferSize) );
		memset( &clientSize, 0, sizeof(clientSize) );
		viewportRect.x = viewportRect.y = 0.0f;
		viewportRect.w = viewportRect.h = 1.0f;
		openWindowTrigger			= -1;
		fullscreenTrigger	= -1;
		maximizeTrigger		= -1;
		deviceActivated		= false;
	}

	void*	device;
	void*	window;
	FRECT32	viewportRect;
	ISIZE32	backBufferSize;
	ISIZE32	clientSize;
	int		openWindowTrigger;
	int		fullscreenTrigger;
	int		maximizeTrigger;
	bool	deviceActivated;

	void ClearTrigger()
	{
		openWindowTrigger = -1;
		fullscreenTrigger = -1;
		maximizeTrigger = -1;
	}
};

class IRenderer
{
	IRenderer( const IRenderer& );
	IRenderer& operator=( const IRenderer& );

public:
	IRenderer() {}
	virtual ~IRenderer() {}

	virtual void SetRenderTextureImmediately( void* renderTexture ) {}
	virtual void SetGraphicsDevice( void* device, int32_t eventType ) {}
	virtual void Render( const RendererParams& rendererParams ) {}
};

} // namespace OVRMirror

#endif // _OVRMIRROR_COMMON_IRENDERER_HPP_
