// Copyright (c) 2014 Nora
// Released under the MIT license
// http://opensource.org/licenses/mit-license.php
#ifndef _OVRMIRROR_COMMON_PRIMITIVE_HPP_
#define _OVRMIRROR_COMMON_PRIMITIVE_HPP_

#include "Config.hpp"

#include <float.h>

namespace OVRMirror
{

#pragma pack(push, 4)
struct Vertex_XYZ_TEX
{
	float pos[3];
	float uv[2];
};
#pragma pack()

class RenderTexturePrimitiveCache
{
public:
	typedef Vertex_XYZ_TEX Vertex;

	Vertex_XYZ_TEX vertices[4];

	RenderTexturePrimitiveCache()
		: _isDirty( true )
	{
		memset( vertices, 0, sizeof(vertices) );
	}

	bool Update( const ISIZE32& clientSize, const FRECT32& viewportRect )
	{
		if( !_isDirty ) {
			_isDirty = ( _clientSize.w != clientSize.w
				|| _clientSize.h != clientSize.h
				|| _viewportRect.x != viewportRect.x
				|| _viewportRect.y != viewportRect.y
				|| _viewportRect.w != viewportRect.w
				|| _viewportRect.h != viewportRect.h );
		}

		if( _isDirty ) {
			_isDirty = false;
			_clientSize = clientSize;
			_viewportRect = viewportRect;

			float viewportRatio = 1.0f, clientRatio = 1.0f;
			if( fabs( viewportRect.h ) > FLT_EPSILON ) {
				viewportRatio = viewportRect.w / viewportRect.h;
			}
			if( clientSize.h != 0 ) {
				clientRatio = (float)clientSize.w / (float)clientSize.h;
			}

			float vX = 1.0f, vY = 1.0f;
			if( viewportRatio < clientRatio ) {
				vY = 1.0f;
				vX *= viewportRatio / clientRatio;
			} else if( viewportRatio > clientRatio ) {
				vX = 1.0f;
				vY *= clientRatio / viewportRatio;
			}

			vertices[0].pos[0] = -vX;
			vertices[0].pos[1] = -vY;
			vertices[1].pos[0] = -vX;
			vertices[1].pos[1] = vY;
			vertices[2].pos[0] = vX;
			vertices[2].pos[1] = -vY;
			vertices[3].pos[0] = vX;
			vertices[3].pos[1] = vY;

			vertices[0].uv[0] = viewportRect.x;
			vertices[0].uv[1] = viewportRect.y;
			vertices[1].uv[0] = viewportRect.x;
			vertices[1].uv[1] = viewportRect.y + viewportRect.h;
			vertices[2].uv[0] = viewportRect.x + viewportRect.w;
			vertices[2].uv[1] = viewportRect.y;
			vertices[3].uv[0] = viewportRect.x + viewportRect.w;
			vertices[3].uv[1] = viewportRect.y + viewportRect.h;
			return true;
		}

		return false;
	}

	void Clear()
	{
		_isDirty = true;
	}

private:
	ISIZE32	_clientSize;
	FRECT32	_viewportRect;
	bool	_isDirty;
};

} // namespace OVRMirror

#endif // _OVRMIRROR_COMMON_PRIMITIVE_HPP_
