// Copyright (c) 2014 Nora
// Released under the MIT license
// http://opensource.org/licenses/mit-license.php
#ifndef _OVRMIRROR_WINDOWS_DX11RENDERER_HPP_
#define _OVRMIRROR_WINDOWS_DX11RENDERER_HPP_

#include "../../Common/IRenderer.hpp"
#include "../../Common/Primitive.hpp"

#include <d3d11.h>

namespace OVRMirror
{
namespace Windows
{

class DX11Renderer : public IRenderer
{
	RecursiveMutex				_mutex;

	bool						_isDeviceInitialized;
	bool						_isDirtyRenderTexture;
	ISIZE32						_backBufferSize;

	ID3D11Resource*				_renderTexture; // memo: Require _mutex.lock()
	ID3D11ShaderResourceView*	_textureView; // memo: Require _mutex.lock()

	ID3D11Buffer*				_vertexBuffer;
	ID3D11VertexShader*			_vertexShader;
	ID3D11PixelShader*			_pixelShader;
	ID3D11InputLayout*			_inputLayout;
	ID3D11RasterizerState*		_rasterState;
	ID3D11BlendState*			_blendState;
	ID3D11DepthStencilState*	_depthState;
	ID3D11SamplerState*			_samplerState;
	IDXGISwapChain*				_swapChain;
	ID3D11Texture2D*			_backBuffer;
	ID3D11RenderTargetView*		_backBufferRT;

	D3D11_VIEWPORT				_backupViewports[D3D11_VIEWPORT_AND_SCISSORRECT_OBJECT_COUNT_PER_PIPELINE];
	ID3D11ClassInstance*		_backupVertexClassInstances[D3D11_SHADER_MAX_INTERFACES];
	ID3D11ClassInstance*		_backupPixelClassInstances[D3D11_SHADER_MAX_INTERFACES];

	RenderTexturePrimitiveCache	_primitiveCache;

public:
	DX11Renderer();
	~DX11Renderer();

	virtual void SetRenderTextureImmediately( void* renderTexture );
	virtual void SetGraphicsDevice( void* device, int32_t eventType );
	virtual void Render( const RendererParams& rendererParams );

private:
	bool _PrepareDevice( const RendererParams& rendererParams );
	void _ShutdownDevice();
};

} // namespace Windows
} // namespace OVRMirror

#endif // _OVRMIRROR_WINDOWS_DX11RENDERER_HPP_
