// Copyright (c) 2014 Nora
// Released under the MIT license
// http://opensource.org/licenses/mit-license.php
#include "Tasktray.hpp"
#include "Window.hpp"
#include "../../Header/OVRMirror.h"
#include <map>

namespace OVRMirror
{
namespace Windows
{

//----------------------------------------------------------------------------------------------------------------------------------------

static HWND _hWnd;

static const wchar_t* _className = L"OVRMirrorTasktray";

//----------------------------------------------------------------------------------------------------------------------------------------

#define WM_TASKTRAY (WM_USER+1)
#define ID_TASKTRAY 1

#define IDM_SHOW 1

//----------------------------------------------------------------------------------------------------------------------------------------

void Tasktray::_CreateWindow()
{
	if( _hWnd == NULL ) {
		const HWND hWndParent = NULL;
		const HINSTANCE hInstance = ::GetModuleHandleW( NULL );

		_RegisterClass();
		
		_hWnd = ::CreateWindowW( _className, L"", WS_POPUP,
				0, 0, 0, 0, hWndParent, NULL, hInstance, NULL );

		ShowWindow( _hWnd, SW_HIDE );
		UpdateWindow( _hWnd );
	}
}

void Tasktray::_DestroyWindow()
{
	if( _hWnd != NULL ) { // Memo: Not use UnsafeCheckHWnd()
		::DestroyWindow( _hWnd );
		_hWnd = NULL;
	}
}

//----------------------------------------------------------------------------------------------------------------------------------------

void Tasktray::_RegisterClass()
{
	HINSTANCE hInstance = ::GetModuleHandleW( NULL );

	WNDCLASSW wc;
	memset( &wc, 0, sizeof(wc) );
	if( ::GetClassInfoW( hInstance, _className, &wc ) != FALSE ) { // If already RegisterClassW(), return TRUE.
		return;
	}

	memset( &wc, 0, sizeof(wc) );
	wc.style         = 0;
	wc.hInstance     = hInstance;
	wc.hCursor       = NULL;
	wc.hIcon         = NULL;
	wc.hbrBackground = NULL;
	wc.lpszClassName = _className;
	wc.lpszMenuName  = NULL;
	wc.lpfnWndProc   = _WndProc;
	wc.cbWndExtra    = 0;
	wc.cbClsExtra    = 0;

	(void)::RegisterClassW( &wc );
}

void Tasktray::_InitializeNotifyIcon( HWND hWnd )
{
	HICON hIcon = Window::_GetRootWindowIcon( true );
	if( hIcon == NULL ) {
		hIcon = LoadIconW( NULL, IDI_APPLICATION );
	}

	NOTIFYICONDATAW nid;
	nid.cbSize				= sizeof( nid );
	nid.hIcon				= hIcon;
	nid.hWnd				= hWnd;
	nid.uCallbackMessage	= WM_TASKTRAY;
	nid.uFlags				= NIF_ICON | NIF_MESSAGE | NIF_TIP;
	nid.uID					= ID_TASKTRAY;
	::wcscpy_s( nid.szTip, 128, L"OVRMirror" );
	Shell_NotifyIconW( NIM_ADD, &nid );
}

void Tasktray::_FinalizeNotifyIcon( HWND hWnd )
{
	NOTIFYICONDATA nid;
	memset( &nid, 0, sizeof( nid ) );
	nid.cbSize	= sizeof( nid );
	nid.hWnd	= hWnd;
	nid.uID		= ID_TASKTRAY;
	Shell_NotifyIcon( NIM_DELETE, &nid );
}

void Tasktray::_ShowTasktrayPopup( HMENU hPopupMenu, HWND hWnd )
{
	POINT pt = { 0, 0 };
	GetCursorPos( &pt );
	TrackPopupMenu( hPopupMenu, TPM_TOPALIGN | TPM_LEFTALIGN | TPM_RIGHTBUTTON, pt.x, pt.y, 0, hWnd, NULL );
}

LRESULT CALLBACK Tasktray::_WndProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
	switch( msg ) {
	case WM_CREATE:
		_InitializeNotifyIcon( hWnd );
		break;
	case WM_TASKTRAY:
		if( wParam == ID_TASKTRAY ) {
			switch( lParam ) {
			case WM_RBUTTONDOWN:
			case WM_LBUTTONDOWN:
				if( OVRMirrorGetWindowState() != 0 ) {
					OVRMirrorCloseWindow();
				} else {
					OVRMirrorOpenWindow(); // Memo: Call OVRMirror API directly.
				}
				break;
			}
			break;
		}
		break;
	case WM_DESTROY:
		_FinalizeNotifyIcon( hWnd );
		break;
	}

	return ::DefWindowProc( hWnd, msg, wParam, lParam );
}

} // namespace Windows
} // namespace OVRMirror
