// Copyright (c) 2014 Nora
// Released under the MIT license
// http://opensource.org/licenses/mit-license.php
#ifndef _OVRMIRROR_WINDOWS_WINDOW_HPP_
#define _OVRMIRROR_WINDOWS_WINDOW_HPP_

#include "../Common/Config.hpp"
#include "../Common/Thread.hpp"

namespace OVRMirror
{
namespace Windows
{

class Window
{
	HWND _hWnd;
	int32_t _deviceType;

public:
	Window();
	~Window();

	RecursiveMutex mutex; // for Unsafe methods.

	HWND GetHWnd()
	{
		LockGuardRecursiveMutex lock(mutex);
		return _hWnd;
	}

	bool CheckHWnd()
	{
		LockGuardRecursiveMutex lock(mutex);
		return UnsafeCheckHWnd();
	}

	bool IsMaximized()
	{
		LockGuardRecursiveMutex lock(mutex);
		return UnsafeIsMaximized();
	}

	bool IsMinimized()
	{
		LockGuardRecursiveMutex lock(mutex);
		return UnsafeIsMinimized();
	}

	bool UnsafeCheckHWnd()
	{
		if( _hWnd != NULL && ::IsWindow( _hWnd ) == FALSE ) {
			_hWnd = NULL;
		}

		return _hWnd != NULL;
	}

	HWND UnsafeGetHWnd() const { return _hWnd; }
	bool UnsafeIsMaximized() { return _hWnd != NULL && ::IsZoomed( _hWnd ) != FALSE; }
	bool UnsafeIsMinimized() { return _hWnd != NULL && ::IsIconic( _hWnd ) != FALSE; }

	void SetDeviceType( int32_t deviceType );

	void _CreateWindow( int32_t deviceType, const ISIZE32& clientSize, bool isFullscreen, bool isMaximized );
	void _DestroyWindow();
	void _ShowWindow();

	void _AdjustWindowSize( const ISIZE32& clientSize );
	void _ChangeWindowStyle_Fullscreen( bool isFullscreen );

	int32_t GetWindowStateImmediately();

	ISIZE32 GetClientSizeImmediately()
	{
		LockGuardRecursiveMutex lock(mutex);
		ISIZE32 r = { 0, 0 };
		if( _hWnd != NULL ) {
			RECT rc = { 0, 0, 0, 0 };
			if( ::GetClientRect( _hWnd, &rc ) != FALSE ) {
				r.w = (int32_t)(rc.right - rc.left);
				r.h = (int32_t)(rc.bottom - rc.top);
			}
		}
		return r;
	}

	static ISIZE32 GetScreenResolution()
	{
		ISIZE32 r = { 0, 0 };
		r.w = (int32_t)::GetSystemMetrics( SM_CXSCREEN );
		r.h = (int32_t)::GetSystemMetrics( SM_CYSCREEN );
		return r;
	}

private:
	static void _RegisterClass();
	static LRESULT CALLBACK _WndProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam );

public: // from Tasktray.cpp
	static HWND _GetRootWindow();
	static HICON _GetRootWindowIcon( bool isSmall );
};

} // namespace Windows
} // namespace OVRMirror

#endif // _OVRMIRROR_WINDOWS_WINDOW_HPP_
